"""
NOTE: This example is from ModernGL 4 or earlier. We simply disable and archive them for now.
"""

# import struct
# import numpy

# import GLWindow
# import ModernGL

# # This example is not working with NPOT Textures

# width, height = 640, 460

# pixels = numpy.round(numpy.random.rand(width, height)).astype('float32')
# grid = numpy.dstack(numpy.mgrid[0:height, 0:width][::-1]).astype('int32')

# wnd = GLWindow.create_window(width, height)
# ctx = ModernGL.create_context()

# prog = ctx.program(
#     ctx.vertex_shader('''
#         #version 330

#         in vec2 vert;
#         out vec2 text;

#         void main() {
#             text = vert;
#             gl_Position = vec4((vert * 2.0 - 1.0) * 1, 0.0, 1.0);
#         }
#     '''),
#     ctx.fragment_shader('''
#         #version 330

#         uniform sampler2D Texture;

#         in vec2 text;
#         out vec4 color;

#         void main() {
#             color = texture(Texture, text);
#         }
#     '''),
# ])

# trans = ctx.program(
#     ctx.vertex_shader('''
#         #version 330

#         uniform sampler2D Texture;
#         uniform int Width;
#         uniform int Height;

#         in ivec2 text;
#         out float vert;

#         #define LIVING 0.0
#         #define DEAD 1.0

#         bool cell(int x, int y) {
#             return texelFetch(Texture, ivec2((x + Width) % Width, (y + Height) % Height), 0).r < 0.5;
#         }

#         void main() {
#             bool living = cell(text.x, text.y);

#             int neighbours = 0;
#             if (cell(text.x - 1, text.y - 1)) neighbours++;
#             if (cell(text.x - 1, text.y + 0)) neighbours++;
#             if (cell(text.x - 1, text.y + 1)) neighbours++;
#             if (cell(text.x + 1, text.y - 1)) neighbours++;
#             if (cell(text.x + 1, text.y + 0)) neighbours++;
#             if (cell(text.x + 1, text.y + 1)) neighbours++;
#             if (cell(text.x + 0, text.y + 1)) neighbours++;
#             if (cell(text.x + 0, text.y - 1)) neighbours++;

#             if (living) {
#                 vert = (neighbours == 2 || neighbours == 3) ? LIVING : DEAD;
#             } else {
#                 vert = (neighbours == 3) ? LIVING : DEAD;
#             }
#         }
#     '''),
#     varyings=['vert']
# )

# trans.uniforms['Width'].value = width
# trans.uniforms['Height'].value = height

# texture = ctx.texture((width, height), 1, pixels.tobytes(), floats=True)
# texture.filter = ModernGL.NEAREST
# texture.swizzle = 'RRR1'
# texture.use()

# vbo = ctx.buffer(struct.pack('8f', 0, 0, 0, 1, 1, 0, 1, 1))
# vao = ctx.simple_vertex_array(prog, vbo, ['vert'])

# text = ctx.buffer(grid.tobytes())
# tao = ctx.simple_vertex_array(trans, text, ['text'])
# pbo = ctx.buffer(reserve=pixels.nbytes)

# while wnd.update():
#     ctx.viewport = wnd.viewport

#     tao.transform(pbo)
#     texture.write(pbo)

#     vao.render(ModernGL.TRIANGLE_STRIP)



'''
    Alpha blending
'''

import numpy as np

import moderngl
from ported._example import Example


class AlphaBlending(Example):
    gl_version = (3, 3)
    title = "Alpha Blending"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.prog = self.ctx.program(
            vertex_shader='''
                #version 330

                in vec2 vert;

                in vec4 vert_color;
                out vec4 frag_color;

                uniform vec2 scale;
                uniform float rotation;

                void main() {
                    frag_color = vert_color;
                    float r = rotation * (0.5 + gl_InstanceID * 0.05);
                    mat2 rot = mat2(cos(r), sin(r), -sin(r), cos(r));
                    gl_Position = vec4((rot * vert) * scale, 0.0, 1.0);
                }
            ''',
            fragment_shader='''
                #version 330
                in vec4 frag_color;
                out vec4 color;
                void main() {
                    color = vec4(frag_color);
                }
            ''',
        )

        self.scale = self.prog['scale']
        self.rotation = self.prog['rotation']

        self.scale.value = (0.5, self.aspect_ratio * 0.5)

        vertices = np.array([
            1.0, 0.0,
            1.0, 0.0, 0.0, 0.5,

            -0.5, 0.86,
            0.0, 1.0, 0.0, 0.5,

            -0.5, -0.86,
            0.0, 0.0, 1.0, 0.5,
        ], dtype='f4')

        self.vbo = self.ctx.buffer(vertices)
        self.vao = self.ctx.simple_vertex_array(self.prog, self.vbo, 'vert', 'vert_color')

    def render(self, time: float, frame_time: float):
        self.ctx.clear(1.0, 1.0, 1.0)
        self.ctx.enable(moderngl.BLEND)
        self.rotation.value = time
        self.vao.render(instances=10)


if __name__ == '__main__':
    AlphaBlending.run()
